@@ -188,7 +188,7 @@ int freenet_generate_KSK (const unsigned char *keystr, freenet_key *key)
   }
 
   key->searchkey[FN_HASH_BYTES] = FN_SVK_PARTSIZE_BYTE;
-  *((uint16_t *)&(key->searchkey[FN_HASH_BYTES+1])) = htons(FN_KSK_TYPE);
+  *((u_int16_t *)&(key->searchkey[FN_HASH_BYTES+1])) = htons(FN_KSK_TYPE);
 
   status = keygen(keystr, keylen, key->enckey, FN_KEY_BYTES);
   if (status != FNS_SUCCESS) {
@@ -220,7 +220,7 @@ int freenet_generate_SVK (unsigned char *priv, const unsigned char *docname,
   }
 
   key->searchkey[FN_HASH_BYTES] = FN_SVK_PARTSIZE_BYTE;
-  *((uint16_t *)&(key->searchkey[FN_HASH_BYTES+1])) = htons(FN_SVK_TYPE);
+  *((u_int16_t *)&(key->searchkey[FN_HASH_BYTES+1])) = htons(FN_SVK_TYPE);
 
   if (strlen(docname) > 0) {
     status = keygen(docname, strlen(docname), key->enckey, FN_KEY_BYTES);
@@ -833,6 +833,8 @@ int freenet_read_stream(freenet_transfer_state *state,
   int partdata;
   int bufpos = 0;
   unsigned char temp_hash[FN_HASH_BYTES];
+  freenet_message message;
+  int i;
 
   if (state->pos + len > state->size) {
     return FNS_OVERREAD;
@@ -904,6 +906,28 @@ int freenet_read_stream(freenet_transfer_state *state,
           return status;
         }
       }
+
+      /* get the storedata message, and make sure it looks okay */
+      status = freenet_recvmsg(&(state->connection), &message);
+      if (status != FNS_SUCCESS) {
+        return status;
+      }
+
+      if (message.uniqueid != state->uniqueid) {
+        return FNS_WRONG_ID;
+      }
+
+      if (message.type != FN_STOREDATA_MSG) {
+        return FNS_UNEXPECTED_MESSAGE;
+      }
+
+      for (i = 0; i < message.numfields; i++) {
+        if (strncmp(message.field[i], "DataSource=", 11) == 0) {
+          strncpy(state->datasource, &(message.field[i][11]),
+                  strlen(&(message.field[i][11])));
+        }
+      }
+
     }
   }
 
@@ -1005,12 +1029,14 @@ int freenet_request_stream (freenet_transfer_state *state, char *address,
     return status;
   }
 
-  status = generate_random((unsigned char *)&(request.uniqueid),
-                           sizeof(uint64_t));
+  status = generate_random((unsigned char *)&(state->uniqueid),
+                           sizeof(u_int64_t));
   if (status != FNS_SUCCESS) {
     return status;
   }
 
+  request.uniqueid = state->uniqueid;
+
   request.type = FN_DATAREQUEST_MSG;
   sprintf(request.field[0], "SearchKey=");
   for (i=0; i<FN_HASH_BYTES + 3; i++) {
@@ -1210,7 +1236,7 @@ int request_insert (freenet_transfer_state *state,
   }
 
   status = generate_random((unsigned char *)&(request.uniqueid),
-                           sizeof(uint64_t));
+                           sizeof(u_int64_t));
   if (status != FNS_SUCCESS) {
     return status;
   }
@@ -1260,21 +1286,14 @@ int request_insert (freenet_transfer_state *state,
 }
 
 
-/* Open a connection to the node at address:port and insert as  CHK
-   len bytes of fbuf (with metadata_len of those bytes as metadata) at
-   hops-to-live htl, with end-to-end encryption scheme endtoend.
-   Fill key with the CHK info.  */
-int freenet_insert_CHK_buffer (freenet_transfer_state *state, char *address,
-                               char *port, int htl, int endtoend,
-                               int metadata_len, freenet_key *key,
-                               unsigned char *fbuf, int len)
+int freenet_generate_CHK_buffer (freenet_transfer_state *state,
+                                 int endtoend, freenet_key *key,
+                                 int *num_parts, unsigned char *fbuf,
+                                 int len)
 {
-  unsigned char header[FN_HEADER_SIZE];
-  int content_len, total_len, logtwo;
-  freenet_message request;
-  int depth;
   int status;
-  int current_part;
+  unsigned char header[FN_HEADER_SIZE];
+  int content_len, logtwo;
 
   key->type = FN_CHK_TYPE;
 
@@ -1303,13 +1322,11 @@ int freenet_insert_CHK_buffer (freenet_transfer_state *state, char *address,
   memcpy(&(header[2]), key->enckey, FN_KEY_BYTES);
 
   status = generate_hashes_from_buffer (state, "", key->enckey, header, fbuf,
-                                        len, &current_part);
+                                        len, num_parts);
   if (status != FNS_SUCCESS) {
     return status;
   }
 
-  total_len = content_len + (current_part * FN_TRAILER_LEN) + 1;
-
   memcpy(key->searchkey, state->hash[0], FN_HASH_BYTES);
 
   /* set the third-to-last byte of the searchkey to the lowest power of two
@@ -1322,7 +1339,36 @@ int freenet_insert_CHK_buffer (freenet_transfer_state *state, char *address,
   }
 
 
-  *((uint16_t *)&(key->searchkey[FN_HASH_BYTES+1])) = htons(FN_CHK_TYPE);
+  *((u_int16_t *)&(key->searchkey[FN_HASH_BYTES+1])) = htons(FN_CHK_TYPE);
+
+  return FNS_SUCCESS;
+}
+
+
+/* Open a connection to the node at address:port and insert as  CHK
+   len bytes of fbuf (with metadata_len of those bytes as metadata) at
+   hops-to-live htl, with end-to-end encryption scheme endtoend.
+   Fill key with the CHK info.  */
+int freenet_insert_CHK_buffer (freenet_transfer_state *state, char *address,
+                               char *port, int htl, int endtoend,
+                               int metadata_len, freenet_key *key,
+                               unsigned char *fbuf, int len)
+{
+  unsigned char header[FN_HEADER_SIZE];
+  freenet_message request;
+  int total_len, content_len;
+  int depth;
+  int num_parts;
+  int status;
+
+  status = freenet_generate_CHK_buffer(state, endtoend, key, &num_parts,
+                                       fbuf, len);
+  if (status != FNS_SUCCESS) {
+    return status;
+  }
+
+  content_len = len + FN_HEADER_SIZE;
+  total_len = content_len + (num_parts * FN_TRAILER_LEN) + 1;
 
   status = freenet_connect(&(state->connection), address, port);
   if (status != FNS_SUCCESS) {
@@ -1342,7 +1388,7 @@ int freenet_insert_CHK_buffer (freenet_transfer_state *state, char *address,
   sprintf(request.field[1], "HopsToLive=%x", htl);
   sprintf(request.field[2], "DataLength=%x", total_len);
   sprintf(request.field[3], "Storable.Metadata-length=%d", metadata_len);
-  if (current_part == 0) {
+  if (num_parts == 0) {
     strcpy(request.field[4], "Storable.PartSize=0");
   } else {
     sprintf(request.field[4], "Storable.PartSize=%x", state->partsize);
@@ -1364,7 +1410,7 @@ int freenet_insert_CHK_buffer (freenet_transfer_state *state, char *address,
   state->pos = 0;
   state->contentpos = 0;
   state->current_part = 0;
-  state->last_part = current_part;
+  state->last_part = num_parts;
 
   status = endtoend_init(state, key->enckey);
   if (status != FNS_SUCCESS) {
@@ -1390,21 +1436,14 @@ int freenet_insert_CHK_buffer (freenet_transfer_state *state, char *address,
 }
 
 
-/* Open a connection to the node at address:port and negotiate the CHK
-   insert of rewindable file stream instream (with metadata_len of
-   bytes of it as metadata) at hops-to-live htl, with end-to-end
-   encryption scheme endtoend.  Fill key with the CHK info. */
-int freenet_insert_CHK_stream (freenet_transfer_state *state, char *address,
-                               char *port, int htl, int endtoend,
-                               int metadata_len, freenet_key *key,
-                               FILE *instream)
+int freenet_generate_CHK_stream (freenet_transfer_state *state,
+                                 int endtoend, freenet_key *key,
+                                 int *num_parts, int *len,
+                                 FILE *instream)
 {
-  unsigned char header[FN_HEADER_SIZE];
-  int len, content_len, total_len, logtwo;
-  freenet_message request;
-  int depth;
   int status;
-  int current_part;
+  unsigned char header[FN_HEADER_SIZE];
+  int content_len, logtwo;
 
   key->type = FN_CHK_TYPE;
 
@@ -1424,8 +1463,8 @@ int freenet_insert_CHK_stream (freenet_transfer_state *state, char *address,
     return FNS_FSEEK_FAILED;
   }
 
-  len = ftell(instream);
-  if (len == -1) {
+  *len = ftell(instream);
+  if (*len == -1) {
     return FNS_FSEEK_FAILED;
   }
 
@@ -1434,7 +1473,7 @@ int freenet_insert_CHK_stream (freenet_transfer_state *state, char *address,
     return FNS_FSEEK_FAILED;
   }
 
-  content_len = len + FN_HEADER_SIZE;
+  content_len = *len + FN_HEADER_SIZE;
 
   if (content_len > FN_PARTSIZE) {
     state->partsize = FN_PARTSIZE;
@@ -1446,14 +1485,12 @@ int freenet_insert_CHK_stream (freenet_transfer_state *state, char *address,
   header[1] = FN_KEY_BYTES;
   memcpy(&(header[2]), key->enckey, FN_KEY_BYTES);
 
-  status = generate_hashes_from_stream (state, "", key->enckey, header, len,
-                                        instream, &current_part);
+  status = generate_hashes_from_stream (state, "", key->enckey, header, *len,
+                                        instream, num_parts);
   if (status != FNS_SUCCESS) {
     return status;
   }
 
-  total_len = content_len + (current_part * FN_TRAILER_LEN) + 1;
-
   memcpy(key->searchkey, state->hash[0], FN_HASH_BYTES);
 
   /* set the third-to-last byte of the searchkey to the lowest power of two
@@ -1465,9 +1502,36 @@ int freenet_insert_CHK_stream (freenet_transfer_state *state, char *address,
     key->searchkey[FN_HASH_BYTES]++;
   }
 
-  *((uint16_t *)&(key->searchkey[FN_HASH_BYTES+1])) = htons(FN_CHK_TYPE);
+  *((u_int16_t *)&(key->searchkey[FN_HASH_BYTES+1])) = htons(FN_CHK_TYPE);
+
+  return FNS_SUCCESS;
+}
 
 
+/* Open a connection to the node at address:port and negotiate the CHK
+   insert of rewindable file stream instream (with metadata_len of
+   bytes of it as metadata) at hops-to-live htl, with end-to-end
+   encryption scheme endtoend.  Fill key with the CHK info. */
+int freenet_insert_CHK_stream (freenet_transfer_state *state, char *address,
+                               char *port, int htl, int endtoend,
+                               int metadata_len, freenet_key *key,
+                               FILE *instream)
+{
+  unsigned char header[FN_HEADER_SIZE];
+  int len, content_len, total_len;
+  freenet_message request;
+  int depth;
+  int status;
+  int num_parts;
+
+  status = freenet_generate_CHK_stream(state, endtoend, key, &num_parts,
+                                       &len, instream);
+  if (status != FNS_SUCCESS) {
+    return status;
+  }
+
+  content_len = len + FN_HEADER_SIZE;
+  total_len = content_len + (num_parts * FN_TRAILER_LEN) + 1;
 
   status = freenet_connect(&(state->connection), address, port);
   if (status != FNS_SUCCESS) {
@@ -1487,7 +1551,7 @@ int freenet_insert_CHK_stream (freenet_transfer_state *state, char *address,
   sprintf(request.field[1], "HopsToLive=%x", htl);
   sprintf(request.field[2], "DataLength=%x", total_len);
   sprintf(request.field[3], "Storable.Metadata-length=%d", metadata_len);
-  if (current_part == 0) {
+  if (num_parts == 0) {
     strcpy(request.field[4], "Storable.PartSize=0");
   } else {
     sprintf(request.field[4], "Storable.PartSize=%x", state->partsize);
@@ -1509,7 +1573,7 @@ int freenet_insert_CHK_stream (freenet_transfer_state *state, char *address,
   state->pos = 0;
   state->contentpos = 0;
   state->current_part = 0;
-  state->last_part = current_part;
+  state->last_part = num_parts;
 
   status = endtoend_init(state, key->enckey);
   if (status != FNS_SUCCESS) {
@@ -1554,7 +1618,7 @@ int freenet_insert_SVK_buffer (freenet_transfer_state *state, char *address,
   freenet_message request;
   int depth;
   int status;
-  int current_part;
+  int num_parts;
   int fieldnum;
 
   if ((endtoend != FN_TWOFISH) &&
@@ -1577,7 +1641,7 @@ int freenet_insert_SVK_buffer (freenet_transfer_state *state, char *address,
   memcpy(&(header[2]), key->enckey, FN_KEY_BYTES);
 
   status = generate_hashes_from_buffer (state, key->docname, key->enckey,
-                                        header, buffer, len, &current_part);
+                                        header, buffer, len, &num_parts);
   if (status != FNS_SUCCESS) {
     return status;
   }
@@ -1690,7 +1754,7 @@ int freenet_insert_SVK_stream (freenet_transfer_state *state, char *address,
   freenet_message request;
   int depth;
   int status;
-  int current_part;
+  int num_parts;
   int fieldnum;
 
   if ((endtoend != FN_TWOFISH) &&
@@ -1729,7 +1793,7 @@ int freenet_insert_SVK_stream (freenet_transfer_state *state, char *address,
 
   status = generate_hashes_from_stream (state, key->docname, key->enckey,
                                         header, len, instream,
-                                        &current_part);
+                                        &num_parts);
   if (status != FNS_SUCCESS) {
     return status;
   }
