@@ -20,7 +20,6 @@
 
 #include <stdlib.h>
 #include <sys/types.h>
-#include <stdint.h>
 
 #include <openssl/sha.h>
 
@@ -129,9 +128,10 @@ typedef struct {
   SHA_CTX hash_handle;
   int datasize;
   int metadata_len;
+  unsigned char datasource[512];
 
   /* insert */
-  uint64_t uniqueid;
+  u_int64_t uniqueid;
   int htl;
 
 } freenet_transfer_state;
@@ -143,25 +143,42 @@ int freenet_generate_SVK (unsigned char *priv, const unsigned char *docname,
 
 int freenet_read_stream(freenet_transfer_state *state,
                         unsigned char *buffer, int len);
+
 int freenet_write_stream(freenet_transfer_state *state,
                          unsigned char *buffer, int len);
+
 int freenet_request_stream (freenet_transfer_state *state, char *address,
                             char *port, int htl, freenet_key *key);
+
 int freenet_request_buffer (freenet_transfer_state *state, char *address,
                             char *port, int htl, freenet_key *key,
                             unsigned char *buffer, int maxlen);
+
+int freenet_generate_CHK_buffer (freenet_transfer_state *state,
+                                 int endtoend, freenet_key *key,
+                                 int *num_parts, unsigned char *fbuf,
+                                 int len);
+
+int freenet_generate_CHK_stream (freenet_transfer_state *state,
+                                 int endtoend, freenet_key *key,
+                                 int *num_parts, int *len,
+                                 FILE *instream);
+
 int freenet_insert_CHK_buffer (freenet_transfer_state *state, char *address,
                                char *port, int htl, int endtoend,
                                int metadata_len, freenet_key *key,
                                unsigned char *fbuf, int len);
+
 int freenet_insert_CHK_stream (freenet_transfer_state *state, char *address,
                                char *port, int htl, int endtoend,
                                int metadata_len, freenet_key *key,
                                FILE *instream);
+
 int freenet_insert_SVK_buffer (freenet_transfer_state *state, char *address,
                                char *port, int htl, int endtoend,
                                int metadata_len, freenet_key *key,
                                unsigned char *buffer, int len);
+
 int freenet_insert_SVK_stream (freenet_transfer_state *state, char *address,
                                char *port, int htl, int endtoend,
                                int metadata_len, freenet_key *key,
