@@ -108,6 +108,14 @@
 #define FNS_BAD_TRAILING_FIELD    -55
 #define FNS_NOT_A_REDIRECT        -56
 #define FNS_INVALID_REDIRECT      -57
+#define FNS_INVALID_64BIT_HEXLEN  -58
+#define FNS_INVALID_HEX_CHAR      -59
+#define FNS_FCNTL_FAILED          -60
+#define FNS_ACCEPT_TIMEDOUT       -61
+#define FNS_MKTIME_FAILED         -62
+#define FNS_NOT_A_MAPFILE         -63
+#define FNS_INVALID_MAPFILE       -64
+#define FNS_DOCNAME_NOT_FOUND     -65
 
 #define FN_STOREDATA_MSG         1
 #define FN_HANDSHAKEREQUEST_MSG  2
@@ -125,7 +133,7 @@
 typedef struct {
   int numfields;
   int type;
-  uint64_t uniqueid;
+  u_int64_t uniqueid;
   char field[128][512];
 } freenet_message;
 
@@ -142,6 +150,7 @@ typedef struct {
   freenet_rijndael_system sendsys;
   freenet_rijndael_system recvsys;
   int expect_callback;
+  time_t timeout;
 } freenet_connection;
 
 int freenet_senddata(freenet_connection *connection, const char *buffer,
@@ -157,4 +166,6 @@ int freenet_connect (freenet_connection *connection, char *hostaddress,
 
 int freenet_get_connection (freenet_connection *connection, u_short port);
 
+int freenet_auth_connection (freenet_connection *connection);
+
 #endif /* __FN_PROTOCOL_H__ */
