@@ -22,6 +22,8 @@
 
 int freenet_parse_request_uri (freenet_key *key, unsigned char *uri);
 int freenet_parse_insert_uri (freenet_key *key, unsigned char *uri);
+int freenet_parse_msk (unsigned char *mskuri, unsigned char *uri,
+                       unsigned char *docname);
 void freenet_get_uri (freenet_key *key, unsigned char *uri);
 
 #endif /* __FN_URI_H__ */
