@@ -24,6 +24,9 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <sys/types.h>
+#include <ctype.h>
+#include <time.h>
 
 #include <openssl/sha.h>
 #include <openssl/dh.h>
@@ -32,6 +35,8 @@
 #include "protocol.h"
 #include "util.h"
 
+int hex2uint64(const char *string, u_int64_t *val);
+
 int readall(int sock, unsigned char *buffer, int len);
 
 int writeall(int sock, unsigned char *buffer, int len);
@@ -39,7 +44,8 @@ int writeall(int sock, unsigned char *buffer, int len);
 int open_connection (freenet_connection *connection, char *hostaddress,
                      char *service);
 
-int get_connection(freenet_connection *connection, u_short port);
+int get_connection(freenet_connection *connection, u_short port,
+                   time_t timeout);
 
 int diffie_hellman (freenet_connection *connection, unsigned char *k,
                     int len);
@@ -57,20 +63,96 @@ int readto(freenet_connection *connection, char *buffer, int buflen, int term);
 int internal_recvmsg (freenet_connection *connection,
                       freenet_message *message);
 
-int freenet_senddata(freenet_connection *connection, const char *buffer,
-                     int len);
-int freenet_sendmsg (freenet_connection *connection, freenet_message *message);
 
-int freenet_readdata(freenet_connection *connection, char *buffer, int len);
+/* Convert a hex string to a 64-bit unsigned int.  I hate my life. */
+int hex2uint64(const char *string, u_int64_t *val)
+{
+  int i;
+  int len;
+  u_int64_t mult[16] = {
+    0x0000000000000001,
+    0x0000000000000010,
+    0x0000000000000100,
+    0x0000000000001000,
+    0x0000000000010000,
+    0x0000000000100000,
+    0x0000000001000000,
+    0x0000000010000000,
+    0x0000000100000000,
+    0x0000001000000000,
+    0x0000010000000000,
+    0x0000100000000000,
+    0x0001000000000000,
+    0x0010000000000000,
+    0x0100000000000000,
+    0x1000000000000000};
 
-int freenet_recvmsg (freenet_connection *connection, freenet_message *message);
+  *val = 0;
 
-int freenet_connect (freenet_connection *connection, char *hostaddress,
-                     char *service);
+  len = strlen(string);
 
-int freenet_get_connection (freenet_connection *connection, u_short port);
+  if ((len < 0) || (len > 16)) {
+    return FNS_INVALID_64BIT_HEXLEN;
+  }
+
+  for (i=0; i<len; i++) {
+    switch (tolower(string[(len-1)-i])) {
+    case '0':
+      break;
+    case '1':
+      *val += (0x1 * mult[i]);
+      break;
+    case '2':
+      *val += (0x2 * mult[i]);
+      break;
+    case '3':
+      *val += (0x3 * mult[i]);
+      break;
+    case '4':
+      *val += (0x4 * mult[i]);
+      break;
+    case '5':
+      *val += (0x5 * mult[i]);
+      break;
+    case '6':
+      *val += (0x6 * mult[i]);
+      break;
+    case '7':
+      *val += (0x7 * mult[i]);
+      break;
+    case '8':
+      *val += (0x8 * mult[i]);
+      break;
+    case '9':
+      *val += (0x9 * mult[i]);
+      break;
+    case 'a':
+      *val += (0xa * mult[i]);
+      break;
+    case 'b':
+      *val += (0xb * mult[i]);
+      break;
+    case 'c':
+      *val += (0xc * mult[i]);
+      break;
+    case 'd':
+      *val += (0xd * mult[i]);
+      break;
+    case 'e':
+      *val += (0xe * mult[i]);
+      break;
+    case 'f':
+      *val += (0xf * mult[i]);
+      break;
+    default:
+      return FNS_INVALID_HEX_CHAR;
+    }
+  }
+
+  return FNS_SUCCESS;
+
+}
 
-int freenet_init (void);
 
 /* just open a tcp connection */
 int open_connection (freenet_connection *connection, char *hostaddress,
@@ -125,7 +207,8 @@ int open_connection (freenet_connection *connection, char *hostaddress,
 
 
 /* wait for a tcp connection */
-int get_connection(freenet_connection *connection, u_short port)
+int get_connection(freenet_connection *connection, u_short port,
+                   time_t timeout)
 {
   struct sockaddr_in address;
   int listening_socket;
@@ -133,6 +216,9 @@ int get_connection(freenet_connection *connection, u_short port)
   int reuse_addr = 1;
   struct sockaddr_in incoming;
   int b;
+  int flags;
+  int status;
+  time_t starttime;
 
   /* Setup internet address information.
      This is used with the bind() call */
@@ -159,17 +245,38 @@ int get_connection(freenet_connection *connection, u_short port)
     return FNS_LISTEN_FAILED;
   }
 
+  if (timeout != 0) {
+    flags = fcntl(listening_socket, F_GETFL);
+    if (flags == -1) {
+      return FNS_FCNTL_FAILED;
+    }
+
+    status = fcntl(listening_socket, F_SETFL, flags | O_NONBLOCK);
+    if (flags == -1) {
+      return FNS_FCNTL_FAILED;
+    }
+  }
+
+  starttime = time(NULL);
   while(connected_socket < 0) {
     b = sizeof(incoming);
     connected_socket = accept(listening_socket, &incoming, &b);
     if (connected_socket < 0) {
       /* Either a real error occured, or blocking was interrupted for
          some reason.  Only abort execution if a real error occured. */
-      if (errno != EINTR) {
+      if (errno == EINTR) {
+        continue;    /* don't quit - do the accept again */
+      } else if ((errno == EAGAIN) && (timeout != 0)) {
+        if ((time(NULL) - starttime) < timeout) {
+          sleep(100);
+          continue;
+        } else {
+          close(listening_socket);
+          return FNS_ACCEPT_TIMEDOUT;
+        }
+      } else {
         close(listening_socket);
         return FNS_ACCEPT_FAILED;
-      } else {
-        continue;    /* don't quit - do the accept again */
       }
     }
 
@@ -380,7 +487,7 @@ int request_handshake (freenet_connection *connection)
 
 
   status = generate_random((unsigned char *)&(message.uniqueid),
-                           sizeof(uint64_t));
+                           sizeof(u_int64_t));
   if (status != FNS_SUCCESS) {
     return status;
   }
@@ -598,7 +705,7 @@ int freenet_sendmsg (freenet_connection *connection, freenet_message *message)
   int i, status;
   unsigned char typeline[512];
   unsigned char uniqline[512];
-
+  u_int32_t luniqueid, runiqueid;
 
   status = msgtype_to_string(message->type, typeline);
   if (status != FNS_SUCCESS) {
@@ -610,20 +717,26 @@ int freenet_sendmsg (freenet_connection *connection, freenet_message *message)
     return status;
   }
 
-  sprintf(uniqline, "UniqueID=%Lx", message->uniqueid);
+  /* sprintf(uniqline, "UniqueID=%Lx", message->uniqueid); */
+  /* all this shifting and weird formatting is here to get the effect of
+     "%Lx" without the GNU extension for long long "L" */
+  luniqueid = (message->uniqueid) / 0x0000000100000000;
+  runiqueid = ((message->uniqueid) * 0x0000000100000000) / 0x0000000100000000;
+  sprintf(uniqline, "UniqueID=%lx%.8lx", luniqueid, runiqueid);
 
   status = writeline(connection, uniqline);
   if (status != FNS_SUCCESS) {
     return status;
   }
 
-  /* DEBUG */
+#ifdef DEBUG
   fprintf(stderr, "sending = \n");
   fprintf(stderr, "%s\n", typeline);
   fprintf(stderr, "%s\n", uniqline);
   for (i = 0; i < message->numfields; i++) {
     fprintf(stderr, "%s\n", message->field[i]);
   }
+#endif
 
   for (i = 0; i < message->numfields; i++) {
     status = writeline(connection, message->field[i]);
@@ -728,10 +841,15 @@ int readto(freenet_connection *connection, char *buffer, int buflen, int term)
 
 int internal_recvmsg (freenet_connection *connection, freenet_message *message)
 {
-  int i, status;
+  int status;
   unsigned char typeline[512];
   unsigned char line[512];
 
+#ifdef DEBUG
+  int i;
+  u_int32_t luniqueid, runiqueid;
+#endif
+
   message->numfields = 0;
   status = readto(connection, typeline, 510, '\n');
   if (status == FNS_EOF) {
@@ -750,7 +868,11 @@ int internal_recvmsg (freenet_connection *connection, freenet_message *message)
     }
 
     if (strncmp(line, "UniqueID=", 9) == 0) {
-      sscanf(&(line[9]), "%Lx", &(message->uniqueid));
+      /* sscanf(&(line[9]), "%Lx", &(message->uniqueid)); */
+      status = hex2uint64(&(line[9]), &(message->uniqueid));
+      if (status != FNS_SUCCESS) {
+        return status;
+      }
     } else if (strchr(line, '=') != NULL) {
       strncpy(message->field[message->numfields], line, 510);
       message->numfields++;
@@ -771,13 +893,19 @@ int internal_recvmsg (freenet_connection *connection, freenet_message *message)
   }
 
 
-  /* DEBUG */
+#ifdef DEBUG
   fprintf(stderr, "received = \n");
   fprintf(stderr, "%s\n", typeline);
-  fprintf(stderr, "UniqueID=%Lx\n", message->uniqueid);
+  /* fprintf(stderr, "UniqueID=%Lx", message->uniqueid); */
+  /* all this shifting and weird formatting is here to get the effect of
+     "%Lx" without the GNU extension for long long "L" */
+  luniqueid = (message->uniqueid) / 0x0000000100000000;
+  runiqueid = ((message->uniqueid) * 0x0000000100000000) / 0x0000000100000000;
+  fprintf(stderr, "UniqueID=%lx%.8lx\n", luniqueid, runiqueid);
   for (i = 0; i < message->numfields; i++) {
     fprintf(stderr, "%s\n", message->field[i]);
   }
+#endif
 
   return FNS_SUCCESS;
 }
@@ -821,34 +949,17 @@ int freenet_recvmsg (freenet_connection *connection, freenet_message *message)
 int freenet_connect (freenet_connection *connection, char *hostaddress,
                      char *service)
 {
-  unsigned char k[FN_DHK_LEN];
-  unsigned char bfmpik[130];
-  unsigned char key[FN_KEY_BYTES];
   int status;
 
   connection->expect_callback = FN_TRUE;
+  connection->timeout = 30;
 
   status = open_connection(connection, hostaddress, service);
   if (status != FNS_SUCCESS) {
     return status;
   }
 
-  status = diffie_hellman(connection, k, FN_DHK_LEN);
-  if (status != FNS_SUCCESS) {
-    return status;
-  }
-
-  status = raw_to_bagbiting_freenet_mpi(k, FN_DHK_LEN, bfmpik);
-  if (status != FNS_SUCCESS) {
-    return status;
-  }
-
-  status = keygen(bfmpik, FN_DHK_MPI_LEN, key, FN_KEY_BYTES);
-  if (status != FNS_SUCCESS) {
-    return status;
-  }
-
-  status = start_ciphers(connection, key);
+  status = freenet_auth_connection(connection);
   if (status != FNS_SUCCESS) {
     return status;
   }
@@ -864,18 +975,31 @@ int freenet_connect (freenet_connection *connection, char *hostaddress,
 
 int freenet_get_connection (freenet_connection *connection, u_short port)
 {
-  unsigned char k[FN_DHK_LEN];
-  unsigned char bfmpik[130];
-  unsigned char key[FN_KEY_BYTES];
   int status;
 
   connection->expect_callback = FN_TRUE;
 
-  status = get_connection(connection, port);
+  status = get_connection(connection, port, connection->timeout);
   if (status != FNS_SUCCESS) {
     return status;
   }
 
+  status = freenet_auth_connection(connection);
+  if (status != FNS_SUCCESS) {
+    return status;
+  }
+
+  return FNS_SUCCESS;
+}
+
+
+int freenet_auth_connection (freenet_connection *connection)
+{
+  unsigned char k[FN_DHK_LEN];
+  unsigned char bfmpik[130];
+  unsigned char key[FN_KEY_BYTES];
+  int status;
+
   status = diffie_hellman(connection, k, FN_DHK_LEN);
   if (status != FNS_SUCCESS) {
     return status;
