@@ -15,11 +15,12 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
-#include <values.h>
+#include <limits.h>
 #include <signal.h>
 #include <string.h>
 #include <time.h>
 #include <unistd.h>
+#include <getopt.h>
 
 #include "protocol.h"
 #include "client.h"
@@ -44,6 +45,8 @@ int main (int argc, char **argv)
   char file[PATH_MAX];
   int htl = 5;
   unsigned char uri[512] = "";
+  unsigned char mskuri[512] = "";
+  unsigned char docname[512] = "";
   unsigned char keystring[512];
   int status;
   freenet_transfer_state state;
@@ -52,7 +55,7 @@ int main (int argc, char **argv)
   unsigned char out;
   FILE *instream;
   unsigned char buffer[66666];
-  unsigned char metadata[1024 + 1];
+  unsigned char metadata[33333 + 1];
   int usebuffer = 0;
   freenet_key key;
   int metadata_len = 0;
@@ -107,7 +110,7 @@ int main (int argc, char **argv)
   }
 
   if (optind < argc) {
-    strcpy(uri, argv[optind]);
+    strncpy(uri, argv[optind], strlen(argv[optind]));
 
     if (cmd == INSERT) {
       status = freenet_parse_insert_uri(&key, uri);
@@ -116,6 +119,12 @@ int main (int argc, char **argv)
         return -1;
       }
     } else {
+      strncpy(mskuri, uri, strlen(uri));
+      status = freenet_parse_msk(mskuri, uri, docname);
+      if (status != FNS_SUCCESS) {
+        fprintf(stderr, "could not parse key: %d\n", status);
+        return -1;
+      }
       status = freenet_parse_request_uri(&key, uri);
       if (status != FNS_SUCCESS) {
         fprintf(stderr, "could not parse key: %d\n", status);
@@ -155,7 +164,7 @@ int main (int argc, char **argv)
           return -1;
         }
 
-        for (i=0; ((i < state.metadata_len) && (i < 1024)); i++) {
+        for (i=0; ((i < state.metadata_len) && (i < 33333)); i++) {
 
           status = freenet_read_stream(&state, &in, 1);
           if (status != FNS_SUCCESS) {
@@ -169,13 +178,24 @@ int main (int argc, char **argv)
 
         metadata[i] = 0;
 
-        status = freenet_parse_redirect(&key, metadata);
-        if ((status != FNS_SUCCESS) && (status != FNS_NOT_A_REDIRECT)) {
-          fprintf(stderr, "redirect handling failed: %d\n", status);
-          return -1;
+        if (state.metadata_len == state.datasize) {
+          status = freenet_parse_redirect(&key, metadata);
+          if ((status != FNS_SUCCESS) && (status != FNS_NOT_A_REDIRECT)) {
+            fprintf(stderr, "redirect handling failed: %d\n", status);
+            return -1;
+          }
+
+          if (status == FNS_NOT_A_REDIRECT) {
+            status = freenet_parse_mapfile(&key, metadata, docname);
+            if ((status != FNS_SUCCESS) && (status != FNS_NOT_A_MAPFILE)) {
+              fprintf(stderr, "redirect handling failed: %d\n", status);
+              return -1;
+            }
+          }
         }
 
-      } while ((status != FNS_NOT_A_REDIRECT) &&
+      } while ((state.metadata_len == state.datasize) &&
+               (status != FNS_NOT_A_MAPFILE) &&
                (redirect_count < max_redirects));
 
       if (show_metadata == 1) {
@@ -212,19 +232,30 @@ int main (int argc, char **argv)
           return -1;
         }
 
-        for (i=0; ((i<state.metadata_len) && (i<1024)); i++) {
+        for (i=0; ((i<state.metadata_len) && (i<33333)); i++) {
           metadata[i] = buffer[i];
         }
 
         metadata[i] = 0;
 
-        status = freenet_parse_redirect(&key, metadata);
-        if ((status != FNS_SUCCESS) && (status != FNS_NOT_A_REDIRECT)) {
-          fprintf(stderr, "redirect handling failed: %d\n", status);
-          return -1;
+        if (state.metadata_len == state.datasize) {
+          status = freenet_parse_redirect(&key, metadata);
+          if ((status != FNS_SUCCESS) && (status != FNS_NOT_A_REDIRECT)) {
+            fprintf(stderr, "redirect handling failed: %d\n", status);
+            return -1;
+          }
+
+          if (status == FNS_NOT_A_REDIRECT) {
+            status = freenet_parse_mapfile(&key, metadata, docname);
+            if ((status != FNS_SUCCESS) && (status != FNS_NOT_A_MAPFILE)) {
+              fprintf(stderr, "redirect handling failed: %d\n", status);
+              return -1;
+            }
+          }
         }
 
-      } while ((status != FNS_NOT_A_REDIRECT) &&
+      } while ((state.metadata_len == state.datasize) &&
+               (status != FNS_NOT_A_MAPFILE) &&
                (redirect_count < max_redirects));
 
       if (show_metadata == 1) {
